from .MakeFunction import(
	WriteFunc1,
	getSinRegression,
	plotRegression
)

from .Algorithm import(
	getPoint,
	getDetedge
)

__all__ = ["WriteFunc1","getSinRegression","plotRegression","getPoint","getDetedge"]